@@ -75,6 +75,11 @@ REGEX is a regular expression that can be used to identify the model, uniquely (
     :capabilities '(generation)
     :context-length 30000
     :regex "o1-mini")
+   (make-llm-model
+    :name "o3 Mini" :symbol 'o3-mini
+    :capabilities '(generation)
+    :context-length 30000
+    :regex "o3-mini")
    (make-llm-model
     :name "GPT-4 Turbo" :symbol 'gpt-4-turbo
     :capabilities '(generation tool-use image-input)
@@ -101,6 +106,11 @@ REGEX is a regular expression that can be used to identify the model, uniquely (
     :context-length 8192
     :regex "text-embedding-ada-002")
    ;; https://docs.anthropic.com/en/docs/about-claude/models
+   (make-llm-model
+    :name "Claude 3.7 Sonnet" :symbol 'claude-3.7-sonnet
+    :capabilities '(generation tool-use image-input pdf-input caching)
+    :context-length 200000
+    :regex "claude-3.7-sonnet")
    (make-llm-model
     :name "Claude 3.5 Sonnet" :symbol 'claude-3.5-sonnet
     :capabilities '(generation tool-use image-input pdf-input caching)
@@ -132,6 +142,16 @@ REGEX is a regular expression that can be used to identify the model, uniquely (
     :capabilities '(generation)
     :context-length 32768
     :regex "gemini-2\\.0-flash-thinking")
+   (make-llm-model
+    :name "Gemini 2.5 Pro" :symbol 'gemini-2.5-pro
+    :capabilities '(generation tool-use image-input audio-input video-input)
+    :context-length 1048576
+    :regex "gemini-2\\.5-pro")
+   (make-llm-model
+    :name "Gemini 2.0 Pro" :symbol 'gemini-2.0-pro
+    :capabilities '(generation tool-use image-input audio-input video-input)
+    :context-length 1048576
+    :regex "gemini-2\\.0-pro")
    (make-llm-model
     :name "Gemini 1.5 Flash" :symbol 'gemini-1.5-flash
     :capabilities '(generation tool-use image-input audio-input video-input)
@@ -194,6 +214,16 @@ REGEX is a regular expression that can be used to identify the model, uniquely (
     :capabilities '(generation free-software)  ;; MIT license
     :context-length 128000
     :regex "deepseek-r1")
+   (make-llm-model
+    :name "deepseek-reasoner" :symbol 'deepseek-reasoner
+    :capabilities '(generation reasoning)
+    :context-length 65536
+    :regex "deepseek-reasoner")
+   (make-llm-model
+    :name "deepseek-chat" :symbol 'deepseek-chat
+    :capabilities '(generation)
+    :context-length 65536
+    :regex "deepseek-chat")
    (make-llm-model
     :name "Mistral" :symbol 'mistral
     :capabilities '(generation tool-use free-software)  ;; Apache license
