@@ -29,6 +29,7 @@
 (require 'llm)
 (require 'llm-provider-utils)
 (require 'llm-models)
+(require 'plz)
 (require 'json)
 (require 'plz-media-type)
 
@@ -104,9 +105,46 @@ PROVIDER is the llm-ollama provider."
 (cl-defmethod llm-provider-batch-embeddings-extract-result ((_ llm-ollama) response)
   (append (assoc-default 'embeddings response) nil))
 
+(eval-and-compile
+  (defconst llm-ollama-reasoning-tags '("think" "reasoning")
+    "A list of possibilities for reasoning tags in Ollama responses.
+
+These are just the text inside the tag, not the tag itself."))
+
 (cl-defmethod llm-provider-chat-extract-result ((_ llm-ollama) response)
   "Return the chat response from the server RESPONSE."
-  (assoc-default 'content (assoc-default 'message response)))
+  (let ((raw-result (assoc-default 'content (assoc-default 'message response))))
+    ;; The raw result may have reasoning content in, which is in <think> tags
+    ;; (for DeepSeek reasoning).  We want to strip that out.
+    (with-temp-buffer
+      (insert raw-result)
+      (goto-char 0)
+      (if (seq-find (lambda (tag)
+                      (search-forward (format "</%s>" tag) nil t))
+                    llm-ollama-reasoning-tags)
+          (string-trim (buffer-substring (point) (point-max)))
+        raw-result))))
+
+(cl-defmethod llm-provider-extract-reasoning ((_ llm-ollama) response)
+  (let ((raw-result (assoc-default 'content (assoc-default 'message response))))
+    ;; Reasoning content is in <think> tags (for DeepSeek reasoning).  We want to
+    ;; extract the content between these tags.
+    (with-temp-buffer
+      (insert raw-result)
+      (goto-char 0)
+      (when (re-search-forward
+             (rx (seq (literal "<")
+                      (group (eval `(or ,@llm-ollama-reasoning-tags)))
+                      (literal ">")))
+             nil t)
+        (when-let* ((end (save-excursion
+                           (re-search-forward
+                            (rx (seq
+                                 (literal "</")
+                                 (group (literal (match-string 1)))
+                                 (literal ">"))) nil t))))
+          ;; +3 to account for the length of the two brackets and slash
+          (buffer-substring (point) (- end (+ 3 (length (match-string 1))))))))))
 
 (defun llm-ollama--response-format (format)
   "Return the response format for FORMAT."
@@ -158,7 +196,13 @@ PROVIDER is the llm-ollama provider."
       (setq options (plist-put options :temperature (llm-chat-prompt-temperature prompt))))
     (when (llm-chat-prompt-max-tokens prompt)
       (setq options (plist-put options :num_predict (llm-chat-prompt-max-tokens prompt))))
-    (setq options (append options (llm-provider-utils-non-standard-params-plist prompt)))
+    (when-let* ((more-options-plist (llm-provider-utils-non-standard-params-plist prompt)))
+      (when-let* ((keep-alive (plist-get more-options-plist :keep_alive)))
+        (setq request-plist (plist-put request-plist :keep_alive keep-alive)))
+      (setq options (append options
+                            (map-into (map-filter (lambda (key _) (not (equal key :keep_alive)))
+                                                  more-options-plist)
+                                      'plist))))
     (when options
       (setq request-plist (plist-put request-plist :options options)))
     request-plist))
@@ -180,14 +224,30 @@ PROVIDER is the llm-ollama provider."
                                                       (llm-provider-utils-tool-use-args tool-use)))))
                     tool-uses))))
 
-(cl-defmethod llm-provider-streaming-media-handler ((_ llm-ollama) msg-receiver _ _)
+(cl-defmethod llm-provider-streaming-media-handler ((_ llm-ollama) receiver _)
   (cons 'application/x-ndjson
         (plz-media-type:application/x-ndjson
-         :handler (lambda (data)
-                    (when-let ((response (assoc-default
-                                          'content
-                                          (assoc-default 'message data))))
-                      (funcall msg-receiver response))))))
+         :handler (let ((in-reasoning))
+                    (lambda (data)
+                      (when-let ((response (assoc-default
+                                            'content
+                                            (assoc-default 'message data))))
+                        ;; The response from ollama should just have the tag and
+                        ;; nothing more.
+                        (cond
+                         ((string-match (rx
+                                         (seq "<"
+                                              (eval `(or ,@llm-ollama-reasoning-tags))
+                                              ">")) response)
+                          (setq in-reasoning t))
+                         ((string-match (rx
+                                         (seq "</"
+                                              (eval `(or ,@llm-ollama-reasoning-tags))
+                                              ">")) response)
+                          (setq in-reasoning nil))
+                         (t (funcall receiver (list (if in-reasoning
+                                                        :reasoning
+                                                      :text) response))))))))))
 
 (cl-defmethod llm-name ((provider llm-ollama))
   (or (llm-ollama-chat-model provider)
@@ -198,7 +258,7 @@ PROVIDER is the llm-ollama provider."
                                         2048))
 
 (cl-defmethod llm-capabilities ((provider llm-ollama))
-  (append '(streaming json-response)
+  (append '(streaming json-response model-list)
           (when (and (llm-ollama-embedding-model provider)
                      (let ((embedding-model (llm-models-match
                                              (llm-ollama-embedding-model provider))))
@@ -209,9 +269,17 @@ PROVIDER is the llm-ollama provider."
                       (chat-model (llm-models-match model))
                       (capabilities (llm-model-capabilities chat-model)))
             (append
-             (when (member 'tool-use capabilities) '(function-calls))
+             (when (member 'tool-use capabilities) '(tool-use))
              (seq-intersection capabilities '(image-input))))))
 
+(cl-defmethod llm-models ((provider llm-ollama))
+  (mapcar (lambda (model-data)
+            (plist-get model-data :name))
+          (plist-get (plz 'get (llm-ollama--url provider "tags")
+                       :as (lambda ()
+                             (json-parse-buffer :object-type 'plist)))
+                     :models )))
+
 (provide 'llm-ollama)
 
 ;;; llm-ollama.el ends here
