@@ -30,6 +30,7 @@
 (require 'llm-provider-utils)
 (require 'llm-models)
 (require 'json)
+(require 'plz)
 (require 'plz-event-source)
 
 (defgroup llm-openai nil
@@ -54,8 +55,8 @@ will use a reasonable default."
   key (chat-model "gpt-4o") (embedding-model "text-embedding-3-small"))
 
 (cl-defstruct (llm-openai-compatible (:include llm-openai
-                                               (chat-model nil)
-                                               (embedding-model nil)))
+                                               (chat-model "unset")
+                                               (embedding-model "unset")))
   "A structure for other APIs that use the Open AI's API.
 
 URL is the URL to use for the API, up to the command.  So, for
@@ -338,7 +339,7 @@ RESPONSE can be nil if the response is complete."
                                  (assoc-default 'tool_calls delta)))))
       content-or-call)))
 
-(cl-defmethod llm-provider-streaming-media-handler ((_ llm-openai) msg-receiver fc-receiver _)
+(cl-defmethod llm-provider-streaming-media-handler ((_ llm-openai) receiver _)
   (cons 'text/event-stream
         (plz-event-source:text/event-stream
          :events `((message
@@ -348,7 +349,10 @@ RESPONSE can be nil if the response is complete."
                          (unless (equal data "[DONE]")
                            (when-let ((response (llm-openai--get-partial-chat-response
                                                  (json-parse-string data :object-type 'alist))))
-                             (funcall (if (stringp response) msg-receiver fc-receiver) response))))))))))
+                             (funcall receiver (if (stringp response)
+                                                   (list :text response)
+                                                 (list :tool-uses-raw
+                                                       response))))))))))))
 
 (cl-defmethod llm-provider-collect-streaming-tool-uses ((_ llm-openai) data)
   (llm-provider-utils-openai-collect-streaming-tool-uses data))
@@ -366,18 +370,28 @@ RESPONSE can be nil if the response is complete."
   (llm-provider-utils-model-token-limit (llm-openai-chat-model provider)))
 
 (cl-defmethod llm-capabilities ((provider llm-openai))
-  (append '(streaming embeddings function-calls json-response)
+  (append '(streaming embeddings tool-use streaming-tool-use json-response model-list)
           (when-let ((model (llm-models-match (llm-openai-chat-model provider))))
             (seq-intersection (llm-model-capabilities model)
                               '(image-input)))))
 
 (cl-defmethod llm-capabilities ((provider llm-openai-compatible))
-  (append '(streaming)
-          (when (llm-openai-embedding-model provider)
+  (append '(streaming model-list)
+          (when (and (llm-openai-embedding-model provider)
+                     (not (equal "unset" (llm-openai-embedding-model provider))))
             '(embeddings embeddings-batch))
-          (let ((model (llm-models-match (llm-openai-chat-model provider))))
-            (when (and model (member 'tool-use (llm-model-capabilities model)))
-              '(function-calls)))))
+          (when-let* ((model (llm-models-match (llm-openai-chat-model provider))))
+            (llm-model-capabilities model))))
+
+(cl-defmethod llm-models ((provider llm-openai))
+  (mapcar (lambda (model)
+            (plist-get model :id))
+          (append
+           (plist-get (plz 'get (llm-openai--url provider "models")
+                        :as (lambda () (json-parse-buffer :object-type 'plist))
+                        :headers (llm-openai--headers provider))
+                      :data)
+           nil)))
 
 (provide 'llm-openai)
 
