@@ -5,7 +5,7 @@
 ;; Author: Andrew Hyatt <ahyatt@gmail.com>
 ;; Homepage: https://github.com/ahyatt/llm
 ;; Package-Requires: ((emacs "28.1") (plz "0.8") (plz-event-source "0.1.1") (plz-media-type "0.2.1"))
-;; Package-Version: 0.23.0
+;; Package-Version: 0.24.2
 ;; SPDX-License-Identifier: GPL-3.0-or-later
 ;;
 ;; This program is free software; you can redistribute it and/or
@@ -342,7 +342,7 @@ need to override it."
   (ignore provider)
   nil)
 
-(cl-defgeneric llm-chat (provider prompt)
+(cl-defgeneric llm-chat (provider prompt &optional multi-output)
   "Return a response to PROMPT from PROVIDER.
 PROMPT is a `llm-chat-prompt'.
 
@@ -352,20 +352,25 @@ conses of the function named called (as a symbol), and the
 corresponding result from calling it.
 
 The prompt's interactions list will be updated to encode the
-conversation so far."
-  (ignore provider prompt)
+conversation so far.
+
+If MULTI-OUTPUT is non-nil the response is a plist with the possible
+keys: `text' (textual output), `reasoning' (reasoning that accompanies
+the output) `tool-uses' (a list of plists with tool `:name' and
+`:args'), and `tool-results' (an alist of results of a calling tools)"
+  (ignore provider prompt multi-output)
   (signal 'not-implemented nil))
 
-(cl-defmethod llm-chat ((_ (eql nil)) _)
+(cl-defmethod llm-chat ((_ (eql nil)) _ &optional _)
   "Catch trivial configuration mistake."
   (error "LLM provider was nil.  Please set the provider in the application you are using"))
 
-(cl-defmethod llm-chat :before (provider _)
+(cl-defmethod llm-chat :before (provider _ &optional _)
   "Issue a warning if the LLM is non-free."
   (when-let (info (llm-nonfree-message-info provider))
     (llm--warn-on-nonfree (llm-name provider) info)))
 
-(cl-defmethod llm-chat :around (provider prompt)
+(cl-defmethod llm-chat :around (provider prompt &optional _)
   "Log the input to llm-chat."
   (llm--log 'api-send :provider provider :prompt prompt)
   ;; We set the debug flag to nil around the next-method so that we don't log
@@ -378,7 +383,7 @@ conversation so far."
       (llm--log 'api-receive :provider provider :msg result))
     result))
 
-(cl-defgeneric llm-chat-async (provider prompt response-callback error-callback)
+(cl-defgeneric llm-chat-async (provider prompt response-callback error-callback &optional multi-output)
   "Call RESPONSE-CALLBACK with a response to PROMPT from PROVIDER.
 
 The response is a string response by the LLM when functions are
@@ -392,6 +397,11 @@ RESPONSE-CALLBACK receives the final text.
 
 ERROR-CALLBACK receives the error response.
 
+If MULTI-OUTPUT is non-nil the response is a plist with the possible
+keys: `text' (textual output), `reasoning' (reasoning that accompanies
+the output) `tool-uses' (a list of plists with tool `:name' and
+`:args'), and `tool-results' (an alist of results of a calling tools)
+
 The prompt's interactions list will be updated to encode the
 conversation so far.
 
@@ -400,14 +410,19 @@ be passed to `llm-cancel-request'."
   ;; By default, you can turn a streaming call into an async call, so we can
   ;; fall back to streaming if async is not populated.
   ;; However, first, we don't want to log twice, so let's delete the last log so that llm-chat-streaming will
-  (llm-chat-streaming provider prompt
-                      ;; Do nothing on partial callback
-                      nil
-                      (lambda (text)
-                        (funcall response-callback text))
-                      (lambda (err msg) (funcall error-callback err msg))))
+  ;;
+  ;; We use `apply' here in case `llm-chat-streaming' is older and doesn't
+  ;; support the multi-output argument.
+  (apply #'llm-chat-streaming
+         provider prompt
+         ;; Do nothing on partial callback
+         nil
+         (lambda (text)
+           (funcall response-callback text))
+         (lambda (err msg) (funcall error-callback err msg))
+         multi-output))
 
-(cl-defmethod llm-chat-async :around (provider prompt response-callback error-callback)
+(cl-defmethod llm-chat-async :around (provider prompt response-callback error-callback &optional multi-output)
   "Log the input to llm-chat-async."
   (llm--log 'api-send :provider provider :prompt prompt)
   (let* ((new-response-callback (lambda (response)
@@ -422,10 +437,11 @@ be passed to `llm-cancel-request'."
          (llm-log nil)
          (result (cl-call-next-method provider prompt
                                       new-response-callback
-                                      new-error-callback)))
+                                      new-error-callback
+                                      multi-output)))
     result))
 
-(cl-defgeneric llm-chat-streaming (provider prompt partial-callback response-callback error-callback)
+(cl-defgeneric llm-chat-streaming (provider prompt partial-callback response-callback error-callback &optional multi-output)
   "Stream a response to PROMPT from PROVIDER.
 PROMPT is a `llm-chat-prompt'.
 
@@ -449,32 +465,33 @@ final text.
 
 ERROR-CALLBACK receives the error response.
 
+If MULTI-OUTPUT is non-nil the response is a plist with the possible
+keys: `text' (textual output), `reasoning' (reasoning that accompanies
+the output) `tool-uses' (a list of plists with tool `:name' and
+`:args'), and `tool-results' (an alist of results of a calling tools)
+
 The prompt's interactions list will be updated to encode the
 conversation so far.
 
 This returns an object representing the async request, which can
 be passed to `llm-cancel-request'."
-  (ignore provider prompt partial-callback response-callback error-callback)
+  (ignore provider prompt partial-callback response-callback error-callback multi-output)
   (signal 'not-implemented nil))
 
-(cl-defmethod llm-chat-streaming ((_ (eql nil)) _ _ _ _)
+(cl-defmethod llm-chat-streaming ((_ (eql nil)) _ _ _ _ &optional _)
   "Catch trivial configuration mistake."
   (error "LLM provider was nil.  Please set the provider in the application you are using"))
 
-(cl-defmethod llm-chat-streaming :before (provider _ _ _ _)
+(cl-defmethod llm-chat-streaming :before (provider _ _ _ _ &optional _)
   "Issue a warning if the LLM is non-free."
   (when-let (info (llm-nonfree-message-info provider))
     (llm--warn-on-nonfree (llm-name provider) info)))
 
-(cl-defmethod llm-chat-streaming :around (provider prompt partial-callback response-callback error-callback)
+(cl-defmethod llm-chat-streaming :around (provider prompt partial-callback response-callback error-callback &optional multi-output)
   "Log the input to llm-chat-async."
   (llm--log 'api-send :provider provider :prompt prompt)
   ;; We need to wrap the callbacks before we set llm-log to nil.
-  (let* ((new-partial-callback (lambda (response)
-                                 (when partial-callback
-                                   (let ((llm-log nil))
-                                     (funcall partial-callback response)))))
-         (new-response-callback (lambda (response)
+  (let* ((new-response-callback (lambda (response)
                                   (llm--log 'api-receive :provider provider :msg response)
                                   (let ((llm-log nil))
                                     (funcall response-callback response))))
@@ -484,15 +501,18 @@ be passed to `llm-cancel-request'."
                                (let ((llm-log nil))
                                  (funcall error-callback type err))))
          (llm-log nil)
-         (result (cl-call-next-method provider prompt new-partial-callback
+         (result (cl-call-next-method provider prompt partial-callback
                                       new-response-callback
-                                      new-error-callback)))
+                                      new-error-callback multi-output)))
     result))
 
-(defun llm-chat-streaming-to-point (provider prompt buffer point finish-callback)
+(cl-defun llm-chat-streaming-to-point (provider prompt buffer point finish-callback
+                                                &key processor)
   "Stream the llm output of PROMPT to POINT in BUFFER.
 PROVIDER is the backend provider of the LLM functionality.
 FINISH-CALLBACK is called with no arguments when the output has finished.
+PROCESSOR, if provided, is a function taking (text position) that processes
+the text before insertion.
 This returns an object representing the async request, which can
 be passed to `llm-cancel-request'."
   (with-current-buffer buffer
@@ -508,10 +528,14 @@ be passed to `llm-cancel-request'."
                     (with-current-buffer (marker-buffer start)
                       (save-excursion
                         (goto-char start)
-                        (let* ((current-text
+                        (let* ((processed-text (if processor
+                                                   (funcall processor text start)
+                                                 text))
+                               (current-text
                                 (buffer-substring-no-properties start end))
                                (common-prefix
-                                (fill-common-string-prefix current-text text))
+                                (fill-common-string-prefix
+                                 current-text processed-text))
                                (prefix-length (length common-prefix)))
                           ;; Skip over common prefix of current text
                           ;; and new text.
@@ -519,18 +543,18 @@ be passed to `llm-cancel-request'."
                             (goto-char (+ start prefix-length)))
                           (delete-region (point) end)
                           ;; Insert new text, minus common prefix.
-                          (insert (substring text prefix-length)))))))
+                          (insert (substring processed-text prefix-length)))))))
           (llm-chat-streaming provider prompt
                               (lambda (text) (insert-text text))
                               (lambda (text) (insert-text text)
                                 (funcall finish-callback))
                               (lambda (_ msg) (error "Error calling the LLM: %s" msg))))))))
 
-(cl-defmethod llm-chat-async ((_ (eql nil)) _ _ _)
+(cl-defmethod llm-chat-async ((_ (eql nil)) _ _ _ &optional _)
   "Catch trivial configuration mistake."
   (error "LLM provider was nil.  Please set the provider in the application you are using"))
 
-(cl-defmethod llm-chat-async :before (provider _ _ _)
+(cl-defmethod llm-chat-async :before (provider _ _ _ &optional _)
   "Issue a warning if the LLM is non-free."
   (when-let (info (llm-nonfree-message-info provider))
     (llm--warn-on-nonfree (llm-name provider) info)))
@@ -552,7 +576,9 @@ won't have any partial responses, so basically just operates like
 
 `embeddings-batch': the LLM can return many vector embeddings at the same time.
 
-`tool-uses': the LLM can call functions.i
+`tool-use': the LLM can call functions.
+
+`streaming-tool-use': The LLM can call functions when streaming.
 
 `image-input': the LLM can accept images as input.
 
@@ -563,7 +589,9 @@ JSON format.
 
 `video-input': the LLM can accept video as input.
 
-`audio-input': the LLM can accept audio as input."
+`audio-input': the LLM can accept audio as input.
+
+`model-list': the provider can return a list of models."
   (ignore provider)
   nil)
 
@@ -675,6 +703,15 @@ methods."
 (cl-defmethod llm-cancel-request ((proc process))
   (delete-process proc))
 
+(cl-defgeneric llm-models (provider)
+  "Return a list of model names for PROVIDER.
+This is not asynchronous, but should be fast.
+
+Not every model provides this, you can check the ones that implement
+`model-list' in `llm-capabilities' before calling."
+  (ignore provider)
+  (signal 'not-implemented nil))
+
 (cl-defgeneric llm-name (_)
   "Return the name of the model in PROVIDER.
 This is expected to be suitable for short labels.  For example, if
